package it.unibz.inf.isochrone.algorithm;

import it.unibz.inf.isochrone.network.Node;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.regex.Pattern;

public class AlgorithmStatistics {
	public enum Statistic {
		LOADED_NODES_CURRENT,
		LOADED_NODES_MAX,
		LOADED_NODES_TOTAL
	}
	/**
	 * The set containing the nodes currently in the memory used by the algorithm.
	 */
	private Collection<Node> nodes;
	/**
	 * Map containing numerical statistics (either with keys from the enum "Statistic" or using strings).
	 */
	private Map<String, Number> numericStats;

	// Getter

	public Collection<Node> getNodes() {
		return nodes;
	}

	public Number getNumericStat(final Statistic s) {
		return (s == null) ? null : getNumericStat(s.name());
	}

	public Number getNumericStat(final String name) {
		if (name == null || numericStats == null) {
			return null;
		}

		final Number n = numericStats.get(name);
		if (n == null) {
			return 0;
		}

		return n;
	}

	public Collection<Number> getNumericStat(final Pattern pattern) {
		if (numericStats == null) {
			return Collections.emptyList();
		}

		final Collection<Number> result = new ArrayList<>();
		final Set<Entry<String, Number>> entries = numericStats.entrySet();
		for (final Entry<String, Number> entry : entries) {
			if (pattern.matcher(entry.getKey()).matches()) {
				result.add(entry.getValue());
			}
		}

		return result;
	}

	// Setter

	public void setNodes(final Collection<Node> nodes) {
		this.nodes = nodes;
	}

	public void setNumericStat(final Statistic s, final Number value) {
		setNumericStat(s.name(), value);
	}

	public void setNumericStat(final String name, final Number value) {
		if (numericStats == null) {
			numericStats = new HashMap<>();
		}

		numericStats.put(name, value);
	}

	// Public methods

	@Override
	public String toString() {
		return "AlgorithmStatistics [nodes=" + ((nodes == null) ? 0 : nodes.size()) + ", numericStats=" + ((numericStats == null) ? 0 : numericStats.size()) + "]";
	}

}
