package it.unibz.inf.isochrone.algorithm;

import it.unibz.inf.isochrone.config.ConfigAlgorithm;
import it.unibz.inf.isochrone.network.AbstractOutput;

import java.io.Serializable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

class CacheKey implements Serializable {
	private static final long serialVersionUID = 6350078085458118454L;
	private static final String PREFIX_DURATION = "duration=";
	private static final int PREFIX_DURATION_LENGTH = PREFIX_DURATION.length();
	private static final Pattern PREFIX_END_PATTERN = Pattern.compile("\\D+");
	private final String algorithmName;
	private final String outputName;
	private final ConfigAlgorithm config;

	// Public comparators

	public static long getDurationFromString(final String str) {
		final int index0 = str.indexOf(PREFIX_DURATION);
		if (index0 < 0) {
			return -1;
		}

		final String tmpStr = str.substring(index0 + PREFIX_DURATION_LENGTH);
		final Matcher matcher = PREFIX_END_PATTERN.matcher(tmpStr);
		final int index1 = (matcher.find()) ? matcher.start() : -1;
		final String durationStr = (index1 < 0) ? tmpStr : tmpStr.substring(0, index1);

		return Long.parseLong(durationStr);
	}

	// Constructor

	public CacheKey(final Class<? extends AbstractAlgorithm> iso, final Class<? extends AbstractOutput> outputClass, final ConfigAlgorithm config) {
		this.algorithmName = iso.getSimpleName();
		this.outputName = outputClass.getSimpleName();
		this.config = config;
	}

	// Public methods

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final CacheKey other = (CacheKey) obj;
		if (algorithmName == null) {
			if (other.algorithmName != null) {
				return false;
			}
		} else if (!algorithmName.equals(other.algorithmName)) {
			return false;
		}
		if (outputName == null) {
			if (other.outputName != null) {
				return false;
			}
		} else if (!outputName.equals(other.outputName)) {
			return false;
		}
		if (config == null) {
			if (other.config != null) {
				return false;
			}
		} else if (!config.equals(other.config)) {
			return false;
		}
		return true;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((algorithmName == null) ? 0 : algorithmName.hashCode());
		result = prime * result + ((outputName == null) ? 0 : outputName.hashCode());
		result = prime * result + ((config == null) ? 0 : config.hashCode());
		return result;
	}

	@Override
	public String toString() {
		return "CacheKey [algorithmName=" + algorithmName + ", outputName=" + outputName + ", config=" + config.toString() + "]";
	}
}
