package it.unibz.inf.isochrone.config;

import com.tngtech.propertyloader.impl.interfaces.PropertyLoaderFilter;

import java.util.Collection;
import java.util.LinkedList;
import java.util.Locale;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.Set;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * Use this filter to prevent the loading of properties which are potentially dangerous for SQL-injectoin.
 */
public class SqlPropertyFilter implements PropertyLoaderFilter {
	private static final Logger LOGGER = LogManager.getLogger(SqlPropertyFilter.class);
	private static String[] suspiciousWords = { ";", "\"", "\'", "/*", "*/", "--", "exec", "select", "update", "delete", "insert", "alter", "drop", "create", "shutdown" };

	public SqlPropertyFilter() {}

	@Override
	public void filter(final Properties properties) {
		findSuspicousKeys(properties).forEach(k -> properties.remove(k));
	}

	// Private methods

	private Collection<Object> findSuspicousKeys(final Properties properties) {
		final Collection<Object> keysToRemove = new LinkedList<Object>();

		final Set<Entry<Object, Object>> s = properties.entrySet();
		for (final Entry<Object, Object> e : s) {
			final Object o = e.getValue();
			if (o == null) {
				continue;
			}

			if (!(o instanceof String)) {
				LOGGER.warn("Invalid datatype in configuration for field \"{}\"... not using configured value", e.getKey());
				keysToRemove.add(e.getKey());
			}

			final String str = ((String) o).toLowerCase(Locale.ENGLISH);
			for (final String susp : suspiciousWords) {
				if (str.contains(susp)) {
					LOGGER.warn("Suspicious value in configuration for field \"{}\"... not using configured value", e.getKey());
					keysToRemove.add(e.getKey());
					break;
				}
			}
		}

		return keysToRemove;
	}

}
