package it.unibz.inf.isochrone.db;

import java.util.HashMap;
import java.util.Map;

abstract class AbstractDbQuery {
	private final String allEdges;
	private final String allEdgesContinuous;
	private final String allSchedules;
	private final String calendar;
	private final String earliestArrivalTimeHomo;
	private final String latestDepartureTimeHomo;
	private final String link;
	private final String linkContinuous;
	private final String linksIncoming;
	private final String linksIncomingContinuous;
	private final String linksInIntersection;
	private final String linksInRange;
	private final String nodeCoordinates;
	private final String projectToLocation;
	private final String rangeByNodeCount;
	private Map<String, Long> stat;

	// Constructor

	protected AbstractDbQuery(final DbQueryBuilder builder) {
		allEdges = builder.allEdges;
		allEdgesContinuous = builder.allEdgesContinuous;
		allSchedules = builder.allSchedules;
		calendar = builder.calendar;
		earliestArrivalTimeHomo = builder.earliestArrivalTimeHomo;
		latestDepartureTimeHomo = builder.latestDepartureTimeHomo;
		link = builder.link;
		linkContinuous = builder.linkContinuous;
		linksIncoming = builder.linksIncoming;
		linksIncomingContinuous = builder.linksIncomingContinuous;
		linksInIntersection = builder.linksInIntersection;
		linksInRange = builder.linksInRange;
		nodeCoordinates = builder.nodeCoordinates;
		projectToLocation = builder.projectToLocation;
		rangeByNodeCount = builder.rangeByNodeCount;
	}

	// Builder

	public static class DbQueryBuilder {
		private String allEdges;
		private String allEdgesContinuous;
		private String allSchedules;
		private String calendar;
		private String earliestArrivalTimeHomo;
		private String latestDepartureTimeHomo;
		private String link;
		private String linkContinuous;
		private String linksIncoming;
		private String linksIncomingContinuous;
		private String linksInIntersection;
		private String linksInRange;
		private String nodeCoordinates;
		private String projectToLocation;
		private String rangeByNodeCount;

		// Setters

		// CHECKSTYLE:OFF HiddenField
		public DbQueryBuilder allEdges(final String allEdges) {
			this.allEdges = allEdges;
			return this;
		}

		public DbQueryBuilder allEdgesContinuous(final String allEdgesContinuous) {
			this.allEdgesContinuous = allEdgesContinuous;
			return this;
		}

		public DbQueryBuilder allSchedules(final String allSchedules) {
			this.allSchedules = allSchedules;
			return this;
		}

		public DbQueryBuilder calendar(final String calendar) {
			this.calendar = calendar;
			return this;
		}

		public DbQueryBuilder earliestArrivalTimeHomo(final String earliestArrivalTimeHomo) {
			this.earliestArrivalTimeHomo = earliestArrivalTimeHomo;
			return this;
		}

		public DbQueryBuilder latestDepartureTimeHomo(final String latestDepartureTimeHomo) {
			this.latestDepartureTimeHomo = latestDepartureTimeHomo;
			return this;
		}

		public DbQueryBuilder link(final String link) {
			this.link = link;
			return this;
		}

		public DbQueryBuilder linkContinuous(final String linkContinuous) {
			this.linkContinuous = linkContinuous;
			return this;
		}

		public DbQueryBuilder links(final String linksIncoming) {
			this.linksIncoming = linksIncoming;
			return this;
		}

		public DbQueryBuilder linksContinuous(final String linksIncomingContinuous) {
			this.linksIncomingContinuous = linksIncomingContinuous;
			return this;
		}

		public DbQueryBuilder linksInIntersection(final String linksInIntersection) {
			this.linksInIntersection = linksInIntersection;
			return this;
		}

		public DbQueryBuilder linksInRange(final String linksInRange) {
			this.linksInRange = linksInRange;
			return this;
		}

		public DbQueryBuilder nodeCoordinates(final String nodeCoordinates) {
			this.nodeCoordinates = nodeCoordinates;
			return this;
		}

		public DbQueryBuilder projectToLocation(final String projectToLocation) {
			this.projectToLocation = projectToLocation;
			return this;
		}

		public DbQueryBuilder rangeByNodeCount(final String rangeByNodeCount) {
			this.rangeByNodeCount = rangeByNodeCount;
			return this;
		}
		// CHECKSTYLE:ON HiddenField
	}

	// Getter

	public String getAllSchedules() {
		incStat("allSchedules");
		return allSchedules;
	}

	public String getEarliestArrivalTimeHomo(final boolean useServices) {
		incStat("earliestArrivalTimeHomo");
		if (!useServices) {
			return earliestArrivalTimeHomo;
		}

		return earliestArrivalTimeHomo + " AND (TIME_D BETWEEN ? AND ?) AND SERVICE_ID IN (%s)";
	}

	public String getLatestDepartureTimeHomo(final boolean useServices) {
		incStat("latestDepartureTimeHomo");
		if (!useServices) {
			return latestDepartureTimeHomo;
		}

		return latestDepartureTimeHomo + " AND (TIME_A BETWEEN ? AND ?) AND SERVICE_ID IN (%s)";
	}

	public String getAllEdges() {
		incStat("allEdges");
		return allEdges;
	}

	public String getAllEdgesContinuous() {
		incStat("allEdgesContinuous");
		return allEdgesContinuous;
	}

	public String getCalendar() {
		incStat("calendar");
		return calendar;
	}

	public String getLink() {
		incStat("link");
		return link;
	}

	public String getLinkContinuous() {
		incStat("linkContinuous");
		return linkContinuous;
	}

	public String getLinks() {
		incStat("links");
		return linksIncoming;
	}

	public String getLinksContinuous() {
		incStat("linksContinuous");
		return linksIncomingContinuous;
	}

	public String getLinksInIntersection() {
		incStat("linksInIntersection");
		return linksInIntersection;
	}

	public String getLinksInRange() {
		incStat("linksInRange");
		return linksInRange;
	}

	public String getNodeCoordinates() {
		incStat("nodeCoordinates");
		return nodeCoordinates;
	}

	public String getRangeByNodeCount() {
		incStat("rangeByNodeCount");
		return rangeByNodeCount;
	}

	public String getProjectToLocation() {
		incStat("projectToLocation");
		return projectToLocation;
	}

	// Public methods

	public void activateStatistics() {
		stat = new HashMap<>();
	}

	public Map<String, Long> getStatistics() {
		return stat;
	}

	// Private methods

	private void incStat(final String str) {
		if (str == null || stat == null) {
			return;
		}

		final String key = "dbQuery_" + str;
		final Long v = stat.get(key);
		stat.put(key, (v == null) ? 1 : v + 1);
	}

}
