package it.unibz.inf.isochrone.db;

import it.unibz.inf.isochrone.config.ConfigDataset;

class DbQueryNeo4j extends AbstractDbQuery {

	// Constructor

	public DbQueryNeo4j(final ConfigDataset config, final boolean isIncoming, final boolean fetchGeometries) {
		super(getDbQueryBuilder(config, isIncoming, fetchGeometries));
	}

	// Private static methods

	private static DbQueryBuilder getDbQueryBuilder(final ConfigDataset config, final boolean isIncoming, final boolean fetchGeometries) {
		final String configEdges = config.getTablenameEdges();
		final String configCalendar = config.getTablenameCalendar();
		final String configSchedule = config.getTablenameSchedule();
		final String configNode = config.getTablenameNode();
		final String configNodeDensity = config.getTablenameNodeDensity();

		final String allEdges = "MATCH (s:Node)-->(n:Edge)-->(t:Node) OPTIONAL MATCH (n)-->(r:Route) return n.id as ID, s.id as SOURCE, t.id as TARGET, n.length as LENGTH, n.edge_mode as EDGE_MODE, r.route_id as ROUTE_ID";

		final String allEdgesContinuous = "MATCH (s:Node)-->(n:Edge {edge_mode: "+Database.NW_MODE_CONTINUOUS+"})-->(t:Node), (n)-->(r:Route) return n.id as ID, s.id as SOURCE, t.id as TARGET, n.length as LENGTH, r.route_id as ROUTE_ID";

		final String calendar = "MATCH (c:Calendar)"
				+ " WHERE ((c.monday AND {1}) OR (c.tuesday AND {2}) OR (c.wednesday AND {3}) OR (c.thursday AND {4}) OR (c.friday AND {5}) OR (c.saturday AND {6}) OR (c.sunday AND {7}))"
				+ " AND c.start_date <= {8} AND c.end_date >= {9}"
				+ " RETURN c.service_id as SERVICE_ID ORDER BY c.service_id";

		final String earliestArrivalTimeHomo = "Match (s:Node {id:{1}})-->(n:Schedule)-->(t:Node {id:{2}}) MATCH (n)-->(r:Route) MATCH (n)-->(c:Calendar) "
				+ "where n.time_a <= {3} and r.route_id IN [%s] %s return n.time_d as TIME_D, n.time_a as TIME_A, r.route_id as ROUTE_ID";
		
		final String latestDepartureTimeHomo = "Match (s:Node {id:{1}})-->(n:Schedule)-->(t:Node {id:{2}}) MATCH (n)-->(r:Route) MATCH (n)-->(c:Calendar) "
				+ "where n.time_d >= {3} and r.route_id IN [%s] %s return n.time_d as TIME_D, n.time_a as TIME_A, r.route_id as ROUTE_ID";
		
		final String link;
		final String linkContinuous;
		if(isIncoming){
			link = "MATCH (t:Node)<--(n:Edge {id:{1}})<--(s:Node) MATCH (s)-[r:EDGE_OUT]->() return n.id as ID, s.id as SOURCE, t.id as TARGET, count(r) as NODE_DEGREE, n.length as LENGTH";
	//		final String link = "SELECT SOURCE, TARGET, "
	//			+ (isIncoming ? "SOURCE_OUTDEGREE" : "TARGET_INDEGREE")
	//			+ " AS NODE_DEGREE, LENGTH FROM " + configEdges
	//			+ " WHERE ID = ?";
			linkContinuous = "MATCH (t:Node)<--(n:Edge {id:{1}, edge_mode:"+Database.NW_MODE_CONTINUOUS+"})<--(s:Node) MATCH (s)-[r:EDGE_OUT]->() return n.id as ID, s.id as SOURCE, t.id as TARGET, count(r) as NODE_DEGREE, n.length as LENGTH";
//			final String linkContinuous = "SELECT SOURCE, TARGET, "
//				+ (isIncoming ? "SOURCE_C_OUTDEGREE" : "TARGET_C_INDEGREE")
//				+ " AS NODE_DEGREE, LENGTH FROM " + configEdges
//				+ " WHERE ID = ? AND EDGE_MODE=" + Database.NW_MODE_CONTINUOUS;
		}else{
			link = "MATCH (s:Node)-->(n:Edge {id:{1}})-->(t:Node) MATCH (t)<-[r:EDGE_IN]-() return n.id as ID, s.id as SOURCE, t.id as TARGET, count(r) as NODE_DEGREE, n.length as LENGTH";
//			final String link = "SELECT SOURCE, TARGET, "
//				+ (isIncoming ? "SOURCE_OUTDEGREE" : "TARGET_INDEGREE")
//				+ " AS NODE_DEGREE, LENGTH FROM " + configEdges
//				+ " WHERE ID = ?";
			linkContinuous = "MATCH (s:Node)-->(n:Edge {id:{1},edge_mode:"+Database.NW_MODE_CONTINUOUS+"})-->(t:Node) MATCH (t)<-[r:EDGE_IN]-() return n.id as ID, s.id as SOURCE, t.id as TARGET, count(r) as NODE_DEGREE, n.length as LENGTH";
//			final String linkContinuous = "SELECT SOURCE, TARGET, "
//				+ (isIncoming ? "SOURCE_C_OUTDEGREE" : "TARGET_C_INDEGREE")
//				+ " AS NODE_DEGREE, LENGTH FROM " + configEdges
//				+ " WHERE ID = ? AND EDGE_MODE=" + Database.NW_MODE_CONTINUOUS;
		}
		
		final String links;
		final String linksContinuous;
		if (isIncoming) {
			links = "MATCH (t:Node {id:{1}})<--(n:Edge)<--(s:Node) MATCH (s)-[r:EDGE_OUT]->() OPTIONAL MATCH (n)-->(route:Route) return n.id as ID, count(r) as NODE_DEGREE, s.id as NODE_ID, n.length as LENGTH, route.route_id as ROUTE_ID, n.edge_mode as EDGE_MODE";
//			links = "SELECT ID, SOURCE NODE_ID, SOURCE_OUTDEGREE NODE_DEGREE, LENGTH, ROUTE_ID, EDGE_MODE FROM "
//				+ configEdges + " WHERE TARGET=?";
			linksContinuous = "MATCH (t:Node {id:{1}})<--(n:Edge {edge_mode:"+Database.NW_MODE_CONTINUOUS+"})<--(s:Node) MATCH (s)-[r:EDGE_OUT]->() OPTIONAL MATCH (n)-->(route:Route) return n.id as ID, count(r) as NODE_DEGREE, s.id as NODE_ID, n.length as LENGTH, route.route_id as ROUTE_ID, n.edge_mode as EDGE_MODE";
//			linksContinuous = "SELECT ID, SOURCE NODE_ID, SOURCE_C_OUTDEGREE NODE_DEGREE, LENGTH, EDGE_MODE FROM "
//				+ configEdges + " WHERE TARGET=? AND EDGE_MODE=" + Database.NW_MODE_CONTINUOUS;
		} else {
			links = "MATCH (s:Node {id:{1}})-->(n:Edge)-->(t:Node) MATCH (t)<-[r:EDGE_IN]-() OPTIONAL MATCH (n)-->(route:Route) return n.id as ID, count(r) as NODE_DEGREE, t.id as NODE_ID, n.length as LENGTH, route.route_id as ROUTE_ID, n.edge_mode as EDGE_MODE";
//			links = "SELECT ID, TARGET NODE_ID, TARGET_INDEGREE NODE_DEGREE, LENGTH, ROUTE_ID, EDGE_MODE FROM "
//				+ configEdges + " WHERE SOURCE=?";
			linksContinuous = "MATCH (s:Node {id:{1}})-->(n:Edge {edge_mode:"+Database.NW_MODE_CONTINUOUS+"})-->(t:Node) MATCH (t)<-[r:EDGE_IN]-() OPTIONAL MATCH (n)-->(route:Route) return n.id as ID, count(r) as NODE_DEGREE, t.id as NODE_ID, n.length as LENGTH, route.route_id as ROUTE_ID, n.edge_mode as EDGE_MODE";
//			linksContinuous = "SELECT ID, TARGET NODE_ID, TARGET_C_INDEGREE NODE_DEGREE, LENGTH, EDGE_MODE FROM "
//				+ configEdges + " WHERE TARGET=? AND EDGE_MODE=" + Database.NW_MODE_CONTINUOUS;
		}
		
		final String linksInIntersection;
		if(isIncoming){
			linksInIntersection = "START t=node:spatial_layer('getNodesInIntersection:[\"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\"]') MATCH (t:Node)<--(n:Edge)<--(s:Node) MATCH (s)-[r:EDGE_OUT]-() OPTIONAL MATCH (n)-->(route:Route) "
					+ "RETURN n.id as ID, s.id AS SOURCE, t.id AS TARGET, n.length as LENGTH, n.edge_mode AS EDGE_MODE, route.route_id AS ROUTE_ID, count(r) AS NODE_DEGREE " + (fetchGeometries ? "n.geometry AS GEOMETRY " : "") + "ORDER BY SOURCE, TARGET";
		}else{
			linksInIntersection = "START s=node:spatial_layer('getNodesInIntersection:[\"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\", \"%s\"]') MATCH (t:Node)<--(n:Edge)<--(s:Node) MATCH (t)-[r:EDGE_IN]-() OPTIONAL MATCH (n)-->(route:Route) "
					+ "RETURN n.id as ID, s.id AS SOURCE, t.id AS TARGET, n.length as LENGTH, n.edge_mode AS EDGE_MODE, route.route_id AS ROUTE_ID, count(r) AS NODE_DEGREE " + (fetchGeometries ? "n.geometry AS GEOMETRY " : "") + "ORDER BY SOURCE, TARGET";
		}
		
		final String linksInRange;
		if(isIncoming){
			linksInRange = "START t=node:spatial_layer('getLinksInRange:[\"%s\", \"%s\", \"%s\"]') MATCH (t:Node)<--(n:Edge)<--(s:Node)  match (s)-[r:EDGE_OUT]-() OPTIONAL MATCH (n)-->(route:Route) "
					+ "RETURN n.id as ID, s.id AS SOURCE, t.id AS TARGET, n.length as LENGTH, n.edge_mode AS EDGE_MODE, route.route_id AS ROUTE_ID, count(r) AS NODE_DEGREE " + (fetchGeometries ? "n.geometry AS GEOMETRY " : "") + "ORDER BY SOURCE, TARGET";
		}else{
			linksInRange = "START s=node:spatial_layer('getLinksInRange:[\"%s\", \"%s\", \"%s\"]') MATCH (t:Node)<--(n:Edge)<--(s:Node) match (t)-[r:EDGE_IN]-() OPTIONAL MATCH (n)-->(route:Route) "
					+ "RETURN n.id as ID, s.id AS SOURCE, t.id AS TARGET, n.length as LENGTH, n.edge_mode AS EDGE_MODE, route.route_id AS ROUTE_ID, count(r) AS NODE_DEGREE " + (fetchGeometries ? "n.geometry AS GEOMETRY " : "") + "ORDER BY SOURCE, TARGET";
		}
		
//		final String linksInRange = "SELECT L.ID, L.SOURCE, L.TARGET, L.LENGTH, L.EDGE_MODE, L.ROUTE_ID, "
//		+ (isIncoming ? "L.SOURCE_OUTDEGREE" : "L.TARGET_INDEGREE") + " NODE_DEGREE"
//		+ ((fetchGeometries) ? ", ST_AsBinary(L.GEOMETRY) AS GEOMETRY" : "")
//		+ " FROM " + configEdges + " L, " + configNode + " N WHERE L." + (isIncoming ? "TARGET" : "SOURCE")
//		+ "=N.ID AND ST_DWITHIN(ST_PointFromText(?, ?), N.GEOMETRY, ?)='TRUE'";

		//TODO check the correctness, use also CastToPoint, but result still null

		final String nodeCoordinates = "START n=node:spatial_layer('getLatLon:[\"%s\", \"%d\"]') return n.lat as X, n.lon as Y";

		final String projectToLocation = "START n=node:spatial_layer('lineLocatePoint:[\"%s\", \"%s\", \"%d\", \"%d\", \"%d\"]') RETURN n.id AS ID, n.offset AS OFFSET ORDER BY n.distance, n.offset limit 2";

		final String rangeByNodeCount = "MATCH (n1:Density {id:{1}}) WHERE n1.density < {2} RETURN \"min\" AS mode, n1.density AS density, n1.e_distance AS e_distance ORDER BY n1.density DESC LIMIT 1 UNION ALL " +
										"MATCH (n1:Density {id:{3}}) WHERE n1.density >= {4} RETURN \"max\" AS mode, n1.density AS density, n1.e_distance AS e_distance ORDER BY n1.density ASC LIMIT 1";
		
//		final String rangeByNodeCount = "SELECT d0.density s0, d0.e_distance ed0, d1.density s1, d1.e_distance ed1"
//				+ " FROM "
//				+ "(SELECT id, density, e_distance FROM " + configNodeDensity
//				+ " WHERE id=? AND density<? ORDER BY density DESC LIMIT 1) d0 NATURAL FULL JOIN "
//				+ "(SELECT id, density, e_distance FROM " + configNodeDensity
//				+ " WHERE id=? AND density>=? ORDER BY density ASC LIMIT 1) d1";

		final String allSchedules = "Match (s:Schedule) return s";

		return new DbQueryBuilder().allEdges(allEdges)
			.allEdgesContinuous(allEdgesContinuous)
			.allSchedules(allSchedules)
			.calendar(calendar)
			.earliestArrivalTimeHomo(earliestArrivalTimeHomo)
			.latestDepartureTimeHomo(latestDepartureTimeHomo)
			.link(link)
			.linkContinuous(linkContinuous)
			.links(links)
			.linksContinuous(linksContinuous)
			.linksInIntersection(linksInIntersection)
			.linksInRange(linksInRange)
			.nodeCoordinates(nodeCoordinates)
			.projectToLocation(projectToLocation)
			.rangeByNodeCount(rangeByNodeCount);
	}

}
