package it.unibz.inf.isochrone.db;

import it.unibz.inf.isochrone.config.ConfigDataset;

class DbQueryPostgres extends AbstractDbQuery {

	// Constructor

	public DbQueryPostgres(final ConfigDataset config, final boolean isIncoming, final boolean fetchGeometries) {
		super(getDbQueryBuilder(config, isIncoming, fetchGeometries));
	}

	// Private static methods

	private static DbQueryBuilder getDbQueryBuilder(final ConfigDataset config, final boolean isIncoming, final boolean fetchGeometries) {
		final String configEdges = config.getTablenameEdges();
		final String configCalendar = config.getTablenameCalendar();
		final String configSchedule = config.getTablenameSchedule();
		final String configNode = config.getTablenameNode();
		final String configNodeDensity = config.getTablenameNodeDensity();

		final String allEdges = "SELECT ID, SOURCE, TARGET, LENGTH, EDGE_MODE, ROUTE_ID"
			+ ((fetchGeometries) ? ", ST_AsBinary(GEOMETRY) AS GEOMETRY" : "")
			+ " FROM " + configEdges;

		final String allEdgesContinuous = "SELECT ID, SOURCE, TARGET, LENGTH, EDGE_MODE"
			+ ((fetchGeometries) ? ", ST_AsBinary(GEOMETRY) AS GEOMETRY" : "")
			+ " FROM " + configEdges + " WHERE EDGE_MODE=" + Database.NW_MODE_CONTINUOUS;

		final String allSchedules = "SELECT * FROM " + configSchedule;

		final String calendar = "SELECT DISTINCT SERVICE_ID "
			+ " FROM " + configCalendar
			+ " WHERE (((MONDAY::int::bit || TUESDAY::int::bit || WEDNESDAY::int::bit || THURSDAY::int::bit"
			+ " || FRIDAY::int::bit || SATURDAY::int::bit || SUNDAY::int::bit) & ?::bit(7))::int) > 0"
			+ " AND START_DATE <= DATE(?) AND END_DATE >= DATE(?) "
			+ " ORDER BY SERVICE_ID";

		final String earliestArrivalTimeHomo = "SELECT TIME_D, TIME_A, ROUTE_ID"
			+ " FROM " + configSchedule
			+ " WHERE SOURCE = ? AND TARGET = ? AND ROUTE_ID IN (%S) AND TIME_A <= ?";

		final String latestDepartureTimeHomo = "SELECT TIME_D, TIME_A, ROUTE_ID"
			+ " FROM " + configSchedule
			+ " WHERE SOURCE = ? AND TARGET = ? AND ROUTE_ID IN (%S) AND TIME_D >= ?";

		final String link = "SELECT SOURCE, TARGET, LENGTH, "
			+ (isIncoming ? "SOURCE_OUTDEGREE" : "TARGET_INDEGREE") + " AS NODE_DEGREE"
			+ ((fetchGeometries) ? ", ST_AsBinary(GEOMETRY) AS GEOMETRY" : "")
			+ " FROM " + configEdges
			+ " WHERE ID = ?";

		final String linkContinuous = "SELECT SOURCE, TARGET, LENGTH, "
			+ (isIncoming ? "SOURCE_C_OUTDEGREE" : "TARGET_C_INDEGREE") + " AS NODE_DEGREE"
			+ ((fetchGeometries) ? ", ST_AsBinary(GEOMETRY) AS GEOMETRY" : "")
			+ " FROM " + configEdges
			+ " WHERE ID = ? AND EDGE_MODE=" + Database.NW_MODE_CONTINUOUS;

		final String links;
		final String linksContinuous;
		if (isIncoming) {
			links = "SELECT ID, SOURCE NODE_ID, SOURCE_OUTDEGREE NODE_DEGREE, LENGTH, ROUTE_ID, EDGE_MODE"
				+ ((fetchGeometries) ? ", ST_AsBinary(GEOMETRY) AS GEOMETRY" : "")
				+ " FROM " + configEdges + " WHERE TARGET=?";
			linksContinuous = "SELECT ID, SOURCE NODE_ID, SOURCE_C_OUTDEGREE NODE_DEGREE, LENGTH, EDGE_MODE"
				+ ((fetchGeometries) ? ", ST_AsBinary(GEOMETRY) AS GEOMETRY" : "")
				+ " FROM " + configEdges + " WHERE TARGET=? AND EDGE_MODE=" + Database.NW_MODE_CONTINUOUS;
		} else {
			links = "SELECT ID, TARGET NODE_ID, TARGET_INDEGREE NODE_DEGREE, LENGTH, ROUTE_ID, EDGE_MODE"
				+ ((fetchGeometries) ? ", ST_AsBinary(GEOMETRY) AS GEOMETRY" : "")
				+ " FROM " + configEdges + " WHERE SOURCE=?";
			linksContinuous = "SELECT ID, TARGET NODE_ID, TARGET_C_INDEGREE NODE_DEGREE, LENGTH, EDGE_MODE"
				+ ((fetchGeometries) ? ", ST_AsBinary(GEOMETRY) AS GEOMETRY" : "")
				+ " FROM " + configEdges + " WHERE TARGET=? AND EDGE_MODE=" + Database.NW_MODE_CONTINUOUS;
		}

		final String linksInIntersection = "SELECT L.ID, L.SOURCE, L.TARGET, L.LENGTH, L.EDGE_MODE, L.ROUTE_ID, "
			+ (isIncoming ? "L.SOURCE_OUTDEGREE" : "L.TARGET_INDEGREE") + " NODE_DEGREE"
			+ ((fetchGeometries) ? ", ST_AsBinary(L.GEOMETRY) AS GEOMETRY" : "")
			+ " FROM " + configEdges + " L, " + configNode + " N WHERE L." + (isIncoming ? "TARGET" : "SOURCE")
			+ "=N.ID AND st_intersects(N.GEOMETRY, %S)";

		final String linksInRange = "SELECT L.ID, L.SOURCE, L.TARGET, L.LENGTH, L.EDGE_MODE, L.ROUTE_ID, "
			+ (isIncoming ? "L.SOURCE_OUTDEGREE" : "L.TARGET_INDEGREE") + " NODE_DEGREE"
			+ ((fetchGeometries) ? ", ST_AsBinary(L.GEOMETRY) AS GEOMETRY" : "")
			+ " FROM " + configEdges + " L, " + configNode + " N WHERE L." + (isIncoming ? "TARGET" : "SOURCE")
			+ "=N.ID AND ST_DWITHIN(ST_PointFromText(?, ?), N.GEOMETRY, ?)='TRUE' ORDER BY L.SOURCE";

		final String nodeCoordinates = "SELECT ID, ST_X(GEOMETRY) X, ST_Y(GEOMETRY) Y"
			+ " FROM " + configNode + " WHERE ID=?";

		final String projectToLocation = "SELECT L.ID, ST_Line_Locate_Point(L.EDGE_GEO, L.POINT_GEO) * L.LENGTH AS OFFSET"
			+ " FROM "
			+ " (SELECT E.ID, E.GEOMETRY EDGE_GEO, E.LENGTH, P.GEOMETRY POINT_GEO FROM %s E,"
			+ " (SELECT %s GEOMETRY) P"
			+ " WHERE E.EDGE_MODE=%s"
			+ " AND ST_DWithin(E.GEOMETRY, P.GEOMETRY, ?)" // tolerance
			+ " ORDER BY ST_Distance(E.GEOMETRY, P.GEOMETRY) LIMIT 2"
			+ ") L";

		final String rangeByNodeCount = "(SELECT 'min' AS mode, density, e_distance FROM " + configNodeDensity
				+ " WHERE id=? AND density<=? ORDER BY density DESC LIMIT 1)"
				+ " UNION"
				+ " (SELECT 'max' AS mode, density, e_distance FROM " + configNodeDensity
				+ " WHERE id=? AND density>=? ORDER BY density ASC LIMIT 1)";

		return new DbQueryBuilder().allEdges(allEdges)
			.allEdgesContinuous(allEdgesContinuous)
			.allSchedules(allSchedules)
			.calendar(calendar)
			.earliestArrivalTimeHomo(earliestArrivalTimeHomo)
			.latestDepartureTimeHomo(latestDepartureTimeHomo)
			.link(link)
			.linkContinuous(linkContinuous)
			.links(links)
			.linksContinuous(linksContinuous)
			.linksInIntersection(linksInIntersection)
			.linksInRange(linksInRange)
			.nodeCoordinates(nodeCoordinates)
			.projectToLocation(projectToLocation)
			.rangeByNodeCount(rangeByNodeCount);
	}

}
