package it.unibz.inf.isochrone.network;

import java.io.Serializable;

/**
 * Represent a position on the link, stored as an offset in respect to the source node.
 *
 * <p>Copyright: 2006 - 2009 <a href="http://www.inf.unibz.it/dis">Dis Research Group</a></p>
 * <p> Domenikanerplatz -  Bozen, Italy.</p>
 *
 * @author Markus Innerebner
 * @version 2.2
 */
public class Location implements Serializable {
	private static final long serialVersionUID = -5904462447962312476L;
	private final int linkId;
	private final double offset;

	// Constructor

	public Location(final int linkId, final double offset) {
		this.linkId = linkId;
		this.offset = offset;
	}

	// Getter

	/**
	 * @return the link id belonging to the location
	 */
	public int getLinkId() {
		return linkId;
	}

	/**
	 * @return the offset in respect to the source node
	 */
	public double getOffset() {
		return offset;
	}

	// Public methods

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final Location other = (Location) obj;
		if (linkId != other.linkId) {
			return false;
		}
		if (Double.doubleToLongBits(offset) != Double.doubleToLongBits(other.offset)) {
			return false;
		}
		return true;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		final int result = prime + linkId;
		final long temp = Double.doubleToLongBits(offset);
		// CHECKSTYLE:OFF MagicNumber
		return prime * result + (int) (temp ^ (temp >>> 32));
		// CHECKSTYLE:ON MagicNumber
	}

	@Override
	public String toString() {
		return "Location [linkId: " + linkId + ", offset: " + offset + "]";
	}

}
