package it.unibz.inf.isochrone.network;

import java.util.Collection;
import java.util.LinkedHashSet;

/**
 * A sample output class, which simply stores all nodes and returns if needed.
 * This is mainly used for testing purposes.
 */
public class MemoryOutput extends AbstractOutput {
	private final Collection<Link> links;
	private final Collection<Node> nodes;

	// Constructor

	public MemoryOutput() {
		links = new LinkedHashSet<>();
		nodes = new LinkedHashSet<>();
	}

	// Getter

	public void clear() {
		links.clear();
		nodes.clear();
	}

	public Collection<Link> getLinks() {
		return links;
	}

	public Collection<Node> getNodes() {
		return nodes;
	}

	// Public methods

	@Override
	public boolean addLink(final Link link) {
		return links.add(link);
	}

	@Override
	public boolean addLinks(final Collection<Link> linkCollection) {
		if (linkCollection == null) {
			return false;
		}

		return links.addAll(linkCollection);
	}

	@Override
	public boolean addNode(final Node node) {
		return nodes.add(node);
	}

	@Override
	public boolean addNodes(final Collection<Node> nodeCollection) {
		if (nodeCollection == null) {
			return false;
		}

		return nodes.addAll(nodeCollection);
	}

	@Override
	public String toString() {
		return "MemoryOutput [#links=" + links.size() + ", #nodes=" + nodes.size() + "]";
	}

}
