package it.unibz.inf.isochrone.network;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

public class NodeConnection implements Serializable {
	private static final long serialVersionUID = -4448803871761333439L;
	private static final int INITIAL_LIST_SIZE = 3;
	private final Node sourceNode;
	private final Map<Node, Collection<Link>> targetRoutes;

	// Collections

	public NodeConnection(final Node sourceNode) {
		this.sourceNode = sourceNode;
		this.targetRoutes = new LinkedHashMap<>();
	}

	// Getters

	public Node getSourceNode() {
		return sourceNode;
	}

	public Map<Node, Collection<Link>> getTargetEntries() {
		return targetRoutes;
	}

	// Public methods

	public void addLink(final Node targetNode, final Link link) {
		addLinkForNode(targetNode, link);
	}

	public boolean containsTargets() {
		return !targetRoutes.isEmpty();
	}

	@Override
	public String toString() {
		return "NodeConnection [sourceNode(id)=" + sourceNode.getId() + "]";
	}

	// Public static methods

	public static Collection<Integer> getTargetRoutes(final Collection<Link> linkSet) {
		final Set<Integer> resultSet = new LinkedHashSet<>(linkSet.size());
		for (final Link l : linkSet) {
			resultSet.add(l.getRoute());
		}
		return resultSet;
	}

	// Private methods

	private void addLinkForNode(final Node node, final Link link) {
		Collection<Link> c = targetRoutes.get(node);
		if (c == null) {
			c = new ArrayList<>(INITIAL_LIST_SIZE);
		}

		c.add(link);
		targetRoutes.put(node, c);
	}

}
