package it.unibz.inf.isochrone.network;

import java.io.Serializable;


/**
 * Represents a Schedule for Transport systems (in memory).
 */
public class Schedule implements Serializable {
	private static final long serialVersionUID = -8345595235671051676L;
	private long arrivalTime = -1;
	private long departureTime = -1;
	private int source = -1;
	private int target = -1;
	private int serviceID = -1;
	private int routeID = -1;

	// Constructors

	public Schedule() { }

	public Schedule(final long departureTime, final long arrivalTime, final int source, final int target, final int serviceID, final int routeID) {
		this.arrivalTime = arrivalTime;
		this.departureTime = departureTime;
		this.source = source;
		this.target = target;
		this.serviceID = serviceID;
		this.routeID = routeID;
	}

	public Schedule(final long departureTime, final long arrivalTime, final int source, final int target) {
		this(departureTime, arrivalTime, source, target, -1, -1);
	}

	public Schedule(final Schedule schedule) {
		if (schedule == null) {
			return;
		}

		this.arrivalTime = schedule.arrivalTime;
		this.departureTime = schedule.departureTime;
		this.source = schedule.source;
		this.target = schedule.target;
		this.serviceID = schedule.serviceID;
		this.routeID = schedule.routeID;
	}

	// Getters

	public long getArrivalTime() {
		return arrivalTime;
	}

	public long getDepartureTime() {
		return departureTime;
	}

	public int getSource() {
		return source;
	}

	public int getServiceID() {
		return serviceID;
	}

	public int getTarget() {
		return target;
	}

	public int getRouteID() {
		return routeID;
	}

	// Setters

	public void setArrivalTime(final long arrivalTime) {
		this.arrivalTime = arrivalTime;
	}

	public void setDepartureTime(final long departureTime) {
		this.departureTime = departureTime;
	}

	public void setSource(final int source) {
		this.source = source;
	}

	public void setServiceID(final int serviceID) {
		this.serviceID = serviceID;
	}

	public void setTarget(final int target) {
		this.target = target;
	}

	public void setRouteID(final int routeID) {
		this.routeID = routeID;
	}

	// Public methods

	@Override
	public String toString() {
		return "Schedule [arrivalTime=" + arrivalTime + ", departureTime="
				+ departureTime + ", source=" + source + ", target=" + target
				+ ", serviceID=" + serviceID + ", routeID=" + routeID + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;

		// CHECKSTYLE:OFF MagicNumber
		result = prime * result + (int) (arrivalTime ^ (arrivalTime >>> 32));
		result = prime * result + (int) (departureTime ^ (departureTime >>> 32));
		result = prime * result + routeID;
		result = prime * result + serviceID;
		result = prime * result + source;
		result = prime * result + target;
		// CHECKSTYLE:ON MagicNumber

		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final Schedule other = (Schedule) obj;
		if (arrivalTime != other.arrivalTime) {
			return false;
		}
		if (departureTime != other.departureTime) {
			return false;
		}
		if (routeID != other.routeID) {
			return false;
		}
		if (serviceID != other.serviceID) {
			return false;
		}
		if (source != other.source) {
			return false;
		}
		if (target != other.target) {
			return false;
		}

		return true;
	}

}
