package it.unibz.inf.isochrone.network;

import java.awt.geom.Point2D;
import java.io.Serializable;


public class Seed implements Serializable {
	private static final long serialVersionUID = -1539388190915934091L;
	private final Point2D center;
	private final double radius;

	// Constructors

	public Seed(final Point2D center, final double radius) {
		this.center = center;
		this.radius = radius;
	}

	// Getter

	public Point2D getCenter() {
		return center;
	}

	public double getRadius() {
		return radius;
	}

	public double euclideanDistance(final Point2D otherCenter) {
		return this.getCenter().distance(otherCenter);
	}

	// Public methods

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final Seed other = (Seed) obj;
		if (center == null) {
			if (other.center != null) {
				return false;
			}
		} else if (!center.equals(other.center)) {
			return false;
		}
		if (Double.doubleToLongBits(radius) != Double.doubleToLongBits(other.radius)) {
			return false;
		}
		return true;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		final int result = prime + ((center == null) ? 0 : center.hashCode());

		final long temp = Double.doubleToLongBits(radius);
		// CHECKSTYLE:OFF MagicNumber
		return prime * result + (int) (temp ^ (temp >>> 32));
		// CHECKSTYLE:ON MagicNumber
	}

	@Override
	public String toString() {
		return "Seed [center=" + center + ", radius=" + radius + "]";
	}

}
