package it.unibz.inf.isochrone;

import it.unibz.inf.isochrone.algorithm.AlgorithmFactory;
import it.unibz.inf.isochrone.config.ConfigAlgorithm.Direction;
import it.unibz.inf.isochrone.config.ConfigAlgorithm.Mode;

import org.kohsuke.args4j.CmdLineException;
import org.testng.annotations.Test;

public class AlgorithmRunnerTest {

	@Test
	public void testDefault() {
		AlgorithmRunner.main(new String[] {});
	}

	@Test
	public void testAll() {
		AlgorithmRunner.main(new String[] {
			"-a", AlgorithmFactory.MINERX.name(),
			"-d", Direction.OUTGOING.name(),
			"-ds", TestConfiguration.DEFAULT_DATASET,
			"-dt", "2011-07-01T09:00",
			"-dmax", "600",
			"-mm", "200",
			"-m", Mode.MULTIMODAL.name(),
			"-q", "11.3525539631585,46.4975205",
			"-ws", "4.2"
		});
	}

	@Test
	public void testAlgorithmParameter() throws Throwable {
		testRunnerParameter(new String[] {
			"-a", AlgorithmFactory.MDIJKSTRA.name()
		});
	}

	@Test
	public void testDirectionParameter() throws Throwable {
		testRunnerParameter(new String[] {
			"-d", Direction.OUTGOING.name()
		});
	}

	@Test
	public void testDatasetParameter() throws Throwable {
		testRunnerParameter(new String[] {
			"-ds", TestConfiguration.DEFAULT_DATASET
		});
	}

	@Test
	public void testDateTimeParameter() throws Throwable {
		testRunnerParameter(new String[] {
			"-dt", "2011-07-01T09:00"
		});
	}

	@Test
	public void testDurationParameter() throws Throwable {
		testRunnerParameter(new String[] {
			"-dmax", "600"
		});
	}

	@Test
	public void testHelpParameter() throws Throwable {
		testRunnerParameter(new String[] {
			"-h"
		});
	}

	@Test
	public void testMaxMemorySizeParameter() throws Throwable {
		testRunnerParameter(new String[] {
			"-a", AlgorithmFactory.MINERX.name(),
			"-mm", "200"
		});
	}

	@Test
	public void testModeParameter() throws Throwable {
		testRunnerParameter(new String[] {
			"-m", Mode.UNIMODAL.name()
		});
	}

	@Test
	public void testQueryPointParameter() throws Throwable {
		testRunnerParameter(new String[] {
			"-q", "11.3525539631585,46.4975205"
		});
	}

	@Test(expectedExceptions = CmdLineException.class)
	public void testQueryPointParameterInvalid() throws Throwable {
		testRunnerParameter(new String[] {
			"-q", "11.35y,46.49x"
		});
	}

	@Test
	public void testWalkingSpeedParameter() throws Throwable {
		testRunnerParameter(new String[] {
			"-ws", "0.5"
		});
	}

	// Private methods

	private void testRunnerParameter(final String[] params) throws Throwable {
		final AlgorithmRunner runner = new AlgorithmRunner(params);

		if (runner.getError() != null) {
			throw runner.getError();
		}
	}

}
