package it.unibz.inf.isochrone;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * Holds information about the execution of a sample test.
 * The information hold is split by isochrone duration and by isochrone algorithm.
 *
 * @author <a href="mailto:nikolaus.krismer@uibk.ac.at">Nikolaus Krismer</a> <nikolaus.krismer@uibk.ac.at>
 */
public class SampleInformation {
	/**
	 * Map holding the sample information.
	 * The key in the first map corresponds to the isochrone maxDuration.
	 * The key in the second map (which is the value in the first map) corresponds to the AbstractAlgorithm class that produces the result.
	 * The key in the third map (which is the value in the second map) corresponds to the name of the dataset under test.
	 * The results are stored as values in the third map. Since results could be computed more than once (e.g.: if 40 test runs were performed),
	 * the results are stored as list.
	 */
	private Map<String, Map<Long, Map<String, List<Double>>>> map;

	// Constructors

	public SampleInformation() {
		map = new LinkedHashMap<>();
	}

	// Getter

	public Map<String, Map<Long, Map<String, List<Double>>>> getInformation() {
		return map;
	}

	// Public methods

	public void add(final String algorithmName, final String datasetName, final double runtime) {
		add(null, algorithmName, datasetName, runtime);
	}

	public void add(final Long duration, final String algorithmName, final String datasetName, final Double runtime) {
		if (!map.containsKey(datasetName)) {
			map.put(datasetName, new LinkedHashMap<Long, Map<String, List<Double>>>());
		}

		final Map<Long, Map<String, List<Double>>> durationMap = map.get(datasetName);
		if (!durationMap.containsKey(duration)) {
			durationMap.put(duration, new LinkedHashMap<>());
		}

		final Map<String, List<Double>> algorithmMap = durationMap.get(duration);
		if (!algorithmMap.containsKey(algorithmName)) {
			algorithmMap.put(algorithmName, new ArrayList<>());
		}

		algorithmMap.get(algorithmName).add(runtime);
	}

}
