package it.unibz.inf.isochrone;

import it.unibz.inf.isochrone.algorithm.AlgorithmStatistics;
import it.unibz.inf.isochrone.algorithm.AlgorithmStatistics.Statistic;

import java.util.Collection;
import java.util.function.Function;
import java.util.regex.Pattern;

public enum SampleProperty {
	NR_NODES_CURRENT("|nodes (memory)|", "nodesCurrent", stat -> stat.getNumericStat(Statistic.LOADED_NODES_CURRENT).longValue()),
	NR_NODES_MAX("|nodes (memory)|", "nodesMax", stat -> stat.getNumericStat(Statistic.LOADED_NODES_MAX).longValue()),
	NR_NODES_TOTAL("|nodes (memory)|", "nodesTotal", stat -> stat.getNumericStat(Statistic.LOADED_NODES_TOTAL).longValue()),
	NR_QUERIES_TOTAL("|queries|", "queriesTotal", stat -> {
		long r = 0;
		final Collection<Number> values = stat.getNumericStat(Pattern.compile("dbQuery\\_(.)*"));
		for (final Number number : values) {
			r += number.longValue();
		}

		return r;
	}),
	NR_QUERIES_EARLIEST_ARRIVAL_TIME("|queries|", "queryEarliestArrivalTime", stat-> {
		return stat.getNumericStat("dbQuery_earliestArrivalTimeHomo").longValue();
	}),
	NR_QUERIES_LATEST_DEPARTURE_TIME("|queries|", "queryLatestDepartureTime", stat-> {
		return stat.getNumericStat("dbQuery_latestDepartureTimeHomo").longValue();
	}),
	RUNTIME("runtime [msec]", null, null);

	private final Function<AlgorithmStatistics, Long> fn;
	private final String unit;
	private final String filenameSuffix;

	// Constructor

	private SampleProperty(final String unit, final String filenameSuffix, final Function<AlgorithmStatistics, Long> fn) {
		this.filenameSuffix = filenameSuffix;
		this.fn = fn;
		this.unit = unit;
	}

	// Getter

	public String getFilenameSuffix() {
		return (filenameSuffix == null) ? "" : "_" + filenameSuffix;
	}

	public long getSamplingInformation(final AlgorithmStatistics stat) {
		return fn.apply(stat);
	}

	public String getUnit() {
		return unit;
	}

};
