package it.unibz.inf.isochrone;

import it.unibz.inf.isochrone.algorithm.AbstractAlgorithm;
import it.unibz.inf.isochrone.config.ConfigAlgorithm;

import java.awt.geom.Point2D;
import java.time.LocalDateTime;
import java.time.Month;

public class TestConfiguration {
	public static final int DEFAULT_MAX_DURATION = 3600; // 1 hours (in sec);
	public static final int DEFAULT_MAX_NODES = 2000;
	/** Default dataset matches dataset of bozen/bolzano. */
	public static final String DEFAULT_DATASET = "bz";
	/** Default location matches address "FUB; Dominikanerplatz". */
	public static final Point2D.Double DEFAULT_QUERYPOINT = new Point2D.Double(11.3525539631585d, 46.4975205d);
	/** Default bus station is located at "Dominikanerplatz" and has nodeId "5026". **/
	public static final Point2D.Double DEFAULT_QUERYPOINT_BUS = new Point2D.Double(11.3516319430908d, 46.49775255d);
	public static final LocalDateTime DEFAULT_DATETIME = LocalDateTime.of(2011, Month.JUNE, 10, 10, 10, 0, 0);

	private final Class<? extends AbstractAlgorithm> algorithmClass;
	private final String algorithmName;
	private final String dataset;
	private final ConfigAlgorithm algorithmConfig;

	public TestConfiguration(final Class<? extends AbstractAlgorithm> algorithmClass, final String dataset, final ConfigAlgorithm algorithmConfig) {
		super();

		this.algorithmClass = algorithmClass;
		this.algorithmConfig = algorithmConfig;
		this.algorithmName = algorithmClass.getSimpleName();
		this.dataset = dataset;
	}

	public TestConfiguration(final Class<? extends AbstractAlgorithm> algorithmClass, final ConfigAlgorithm algorithmConfig) {
		this(algorithmClass, DEFAULT_DATASET, algorithmConfig);
	}

	// Getters

	public Class<? extends AbstractAlgorithm> getAlgorithmClass() {
		return algorithmClass;
	}

	public ConfigAlgorithm getAlgorithmConfiguration() {
		return algorithmConfig;
	}

	public String getDataset() {
		return dataset;
	}

	// Public methods

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final TestConfiguration other = (TestConfiguration) obj;
		if (algorithmName == null) {
			if (other.algorithmName != null) {
				return false;
			}
		} else if (!algorithmName.equals(other.algorithmName)) {
			return false;
		}
		if (dataset == null) {
			if (other.dataset != null) {
				return false;
			}
		} else if (!dataset.equals(other.dataset)) {
			return false;
		}
		if (algorithmConfig == null) {
			if (other.algorithmConfig != null) {
				return false;
			}
		} else if (!algorithmConfig.equals(other.algorithmConfig)) {
			return false;
		}
		return true;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((algorithmName == null) ? 0 : algorithmName.hashCode());
		result = prime * result + ((dataset == null) ? 0 : dataset.hashCode());
		result = prime * result + ((algorithmConfig == null) ? 0 : algorithmConfig.hashCode());
		return result;
	}

	@Override
	public String toString() {
		return "TestConfiguration [algorithmName=" + algorithmName + ", dataset=" + dataset + ", duration=" + algorithmConfig.getDuration() + "]";
	}

}
