package it.unibz.inf.isochrone.algorithm;

import it.unibz.inf.isochrone.TestConfiguration;
import it.unibz.inf.isochrone.TestHelper;
import it.unibz.inf.isochrone.TestHelper.AlgorithmResult;
import it.unibz.inf.isochrone.algorithm.AlgorithmStatistics.Statistic;
import it.unibz.inf.isochrone.config.ConfigAlgorithm;
import it.unibz.inf.isochrone.config.ConfigAlgorithm.ConfigAlgorithmBuilder;
import it.unibz.inf.isochrone.config.ConfigDataset;
import it.unibz.inf.isochrone.network.MemoryOutput;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

public class ExpirationTest {
	private static final Logger LOGGER = LogManager.getLogger(ExpirationTest.class);
	private static final ConfigAlgorithm TEST_CONFIG = new ConfigAlgorithmBuilder(TestConfiguration.DEFAULT_QUERYPOINT).duration(TestConfiguration.DEFAULT_MAX_DURATION).build();
	private static final ConfigAlgorithm TEST_CONFIG_CACHED = new ConfigAlgorithmBuilder(TestConfiguration.DEFAULT_QUERYPOINT).duration(TestConfiguration.DEFAULT_MAX_DURATION).useCache().build();

	// Public methods

	@BeforeClass
	public void setup() {
		LOGGER.info("Starting test(s) in class \"{}\"", ExpirationTest.class.getSimpleName());
		// loads dataset configurations (we do this before testing, so timings are correct)
		ConfigDataset.getInstance(TestConfiguration.DEFAULT_DATASET);
		LOGGER.info("-----------------------------");
	}

	// Test methods

	@Test
	public void testMineLinksInOutput() {
		final int nrLinks = run(Mine.class).getOutput().getLinks().size();
		final int nrLinksExpired = run(MineX.class).getOutput().getLinks().size();

		Assert.assertEquals(nrLinks, nrLinksExpired, "It seems like links were missing in output when using MineX algorithm [nrLinks=" + nrLinks + ";nrLinksExpired=" + nrLinksExpired + "]");
	}

	@Test
	public void testMineNodesInMemory() {
		// we can not use caching here... otherwise 0 nodes were loaded (loaded nodes are not cached)
		final long nrNodes = run(Mine.class, false).getStatistics().getNumericStat(Statistic.LOADED_NODES_CURRENT).longValue();
		final long nrNodesExpired = run(MineX.class, false).getStatistics().getNumericStat(Statistic.LOADED_NODES_CURRENT).longValue();

		Assert.assertTrue(nrNodes > nrNodesExpired, "It seems like no nodes were expired when using MineX algorithm [nrNodes=" + nrNodes + ";nrNodesExpired=" + nrNodesExpired + "]");
	}

	@Test
	public void testMineNodesInOutput() {
		final int nrNodes = run(Mine.class).getOutput().getNodes().size();
		final int nrNodesExpired = run(MineX.class).getOutput().getNodes().size();

		Assert.assertEquals(nrNodes, nrNodesExpired, "It seems like nodes were missing in output when using MineX algorithm [nrNodes=" + nrNodes + ";nrNodesExpired=" + nrNodesExpired + "]");
	}

	@Test
	public void testMineRLinksInOutput() {
		final int nrNodes = run(MineR.class).getOutput().getLinks().size();
		final int nrNodesExpired = run(MineRX.class).getOutput().getLinks().size();

		Assert.assertEquals(nrNodes, nrNodesExpired, "It seems like links were missing in output when using MineRX algorithm [nrNodes=" + nrNodes + ";nrNodesExpired=" + nrNodesExpired + "]");
	}

	@Test
	public void testMineRNodesInMemory() {
		// we can not use caching here... otherwise 0 nodes were loaded (loaded nodes are not cached)
		final long nrNodes = run(MineR.class, false).getStatistics().getNumericStat(Statistic.LOADED_NODES_CURRENT).longValue();
		final long nrNodesExpired = run(MineRX.class, false).getStatistics().getNumericStat(Statistic.LOADED_NODES_CURRENT).longValue();

		Assert.assertTrue(nrNodes > nrNodesExpired, "It seems like no nodes were expired when using MineRX algorithm [nrNodes=" + nrNodes + ";nrNodesExpired=" + nrNodesExpired + "]");
	}

	@Test
	public void testMineRNodesInOutput() {
		final int nrNodes = run(MineR.class).getOutput().getNodes().size();
		final int nrNodesExpired = run(MineRX.class).getOutput().getNodes().size();

		Assert.assertEquals(nrNodes, nrNodesExpired, "It seems like nodes were missing in output when using MineRX algorithm [nrNodes=" + nrNodes + ";nrNodesExpired=" + nrNodesExpired + "]");
	}

	// Private static methods

	private static AlgorithmResult<MemoryOutput> run(final Class<? extends AbstractAlgorithm> algorithmClass) {
		return run(algorithmClass, true);
	}

	private static AlgorithmResult<MemoryOutput> run(final Class<? extends AbstractAlgorithm> algorithmClass, final boolean useCache) {
		LOGGER.info("Executing \"{}\"", algorithmClass.getSimpleName());
		return TestHelper.runQuietly(new TestConfiguration(algorithmClass, (useCache) ? TEST_CONFIG_CACHED : TEST_CONFIG));
	}

}
