package it.unibz.inf.isochrone.algorithm;

import it.unibz.inf.isochrone.SamplePlotter;
import it.unibz.inf.isochrone.SamplePlotter.Aggregator;
import it.unibz.inf.isochrone.SampleProperty;
import it.unibz.inf.isochrone.SampleRunner;
import it.unibz.inf.isochrone.SampleRunner.SampleRunnerBuilder;
import it.unibz.inf.isochrone.TestConfiguration;
import it.unibz.inf.isochrone.TestHelper;
import it.unibz.inf.isochrone.config.ConfigAlgorithm.ConfigAlgorithmBuilder;
import it.unibz.inf.isochrone.config.ConfigAlgorithm.Mode;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Collection;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

public class MemoryTest {
	private static final Logger LOGGER = LogManager.getLogger(MemoryTest.class);
	private static final int GRID_CENTER_LENGTH = 3000;
	private static final String RESULTFILE_NAME = "build/reports/tests/algorithm-memory";
	private static final int SRID_SYNTHETIC = 3857;
	private static final String TEST_DATASET_GRID = "grid_s100";
	private static final String TEST_DATASET_SPIDER = "spider_l1000";
	private SampleRunner sampleRunner;

	// Public methods

	@BeforeClass
	public void setup() {
		LOGGER.info("Starting test(s) in class \"{}\"", MemoryTest.class.getSimpleName());

		final Collection<SampleProperty> sampleProperties = new ArrayList<>(3);
		sampleProperties.add(SampleProperty.NR_NODES_CURRENT);
		sampleProperties.add(SampleProperty.NR_NODES_MAX);
		sampleProperties.add(SampleProperty.NR_NODES_TOTAL);

		final Point2D.Double pGridCentral = new Point2D.Double(GRID_CENTER_LENGTH, GRID_CENTER_LENGTH);
		final Point2D.Double pSpiderCentral = new Point2D.Double(0, 0);
		final ConfigAlgorithmBuilder configDefault = new ConfigAlgorithmBuilder(TestConfiguration.DEFAULT_QUERYPOINT).dateTime(TestConfiguration.DEFAULT_DATETIME);
		final ConfigAlgorithmBuilder configGrid = new ConfigAlgorithmBuilder(pGridCentral).queryPointSRID(SRID_SYNTHETIC).mode(Mode.UNIMODAL);
		final ConfigAlgorithmBuilder configSpider = new ConfigAlgorithmBuilder(pSpiderCentral).queryPointSRID(SRID_SYNTHETIC).mode(Mode.UNIMODAL);

		sampleRunner = new SampleRunnerBuilder(sampleProperties)
			.addDataset(TestConfiguration.DEFAULT_DATASET, configDefault)
			.addDataset(TEST_DATASET_GRID, configGrid)
			.addDataset(TEST_DATASET_SPIDER, configSpider)
			.addMaxDuration(TestConfiguration.DEFAULT_MAX_DURATION)
			.addMaxNodes(TestConfiguration.DEFAULT_MAX_NODES)
			.build();

		LOGGER.info("-----------------------------");
	}

	@AfterClass
	public void writeRuntimes() {
		new SamplePlotter(Aggregator.FIRST, RESULTFILE_NAME, sampleRunner, SampleProperty.NR_NODES_CURRENT).writeAll();
		new SamplePlotter(Aggregator.FIRST, RESULTFILE_NAME, sampleRunner, SampleProperty.NR_NODES_MAX).writeAll();
		new SamplePlotter(Aggregator.FIRST, RESULTFILE_NAME, sampleRunner, SampleProperty.NR_NODES_TOTAL).writeAll();
	}

	// Test methods

	@Test
	public void testWithDefaultParameters() {
		LOGGER.info("Running tests with default test parameters");
		LOGGER.info("-----------------------------");
		final Collection<Class<? extends AbstractAlgorithm>> algorithms = TestHelper.getAllAlgorithms();
		sampleRunner.sample(TestHelper::runQuietly, algorithms, LOGGER);
	}

}
