package it.unibz.inf.isochrone.algorithm;

import it.unibz.inf.isochrone.SamplePlotter;
import it.unibz.inf.isochrone.SamplePlotter.Aggregator;
import it.unibz.inf.isochrone.SampleProperty;
import it.unibz.inf.isochrone.SampleRunner;
import it.unibz.inf.isochrone.SampleRunner.SampleRunnerBuilder;
import it.unibz.inf.isochrone.TestConfiguration;
import it.unibz.inf.isochrone.TestHelper;

import java.util.Arrays;
import java.util.Collection;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

public class QueryQuantifierTest {
	private static final Logger LOGGER = LogManager.getLogger(QueryQuantifierTest.class);
	private static final String RESULTFILE_NAME = "build/reports/tests/algorithm-querycount";
	private static final Collection<SampleProperty> SAMPLE_PROPERTIES = Arrays.asList(new SampleProperty[] { SampleProperty.NR_QUERIES_TOTAL, SampleProperty.NR_QUERIES_LATEST_DEPARTURE_TIME});

	private SampleRunner sampleRunner;

	// Public methods

	@BeforeClass
	public void setup() {
		LOGGER.info("Starting test(s) in class \"{}\"", QueryQuantifierTest.class.getSimpleName());
		sampleRunner = new SampleRunnerBuilder(SAMPLE_PROPERTIES)
			.addMaxDuration(TestConfiguration.DEFAULT_MAX_DURATION)
			.build();

		LOGGER.info("-----------------------------");
	}

	@AfterClass
	public void writeRuntimes() {
		for (final SampleProperty sampleProperty : SAMPLE_PROPERTIES) {
			new SamplePlotter(Aggregator.FIRST, RESULTFILE_NAME, sampleRunner, sampleProperty).writeAll();
		}
	}

	// Test methods

	@Test
	public void testWithDefaultParameters() {
		LOGGER.info("Running tests with default test parameters");
		LOGGER.info("-----------------------------");
		final Collection<Class<? extends AbstractAlgorithm>> algorithms = TestHelper.getAllAlgorithms();
		sampleRunner.sample(TestHelper::runQuietly, algorithms, LOGGER);
	}

}
