package it.unibz.inf.isochrone.algorithm;

import it.unibz.inf.isochrone.SamplePlotter;
import it.unibz.inf.isochrone.SamplePlotter.Aggregator;
import it.unibz.inf.isochrone.SampleProperty;
import it.unibz.inf.isochrone.SampleRunner;
import it.unibz.inf.isochrone.SampleRunner.SampleRunnerBuilder;
import it.unibz.inf.isochrone.TestConfiguration;
import it.unibz.inf.isochrone.TestHelper;

import java.util.Collection;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

/**
 * Sophisticated performance test that executes each algorithm
 * and computes different isochrones multiple times.
 */
// TODO: Activate tests for more datasets and plot results for each one (into separate files)
public final class RuntimeTest {
	// General properties
	private static final Logger LOGGER = LogManager.getLogger(RuntimeTest.class);
	private static final int REPEAT_COUNT = 10;
	private static final String RESULTFILE_NAME = "build/reports/tests/algorithm-runtimes";
	private static final SampleProperty SAMPLE_PROPERTY = SampleProperty.RUNTIME;

	private int currentTestCount = 0;
	private SampleRunner sampleRunner;

	// Public methods

	@BeforeClass(alwaysRun = true)
	public void setup() {
		LOGGER.info("Starting test(s) in class \"{}\"", RuntimeTest.class.getSimpleName());
		sampleRunner = new SampleRunnerBuilder(SAMPLE_PROPERTY)
			.addMaxDuration(TestConfiguration.DEFAULT_MAX_DURATION)
			.build();

		LOGGER.info("-----------------------------");
	}

	@BeforeMethod(alwaysRun = true)
	public void prepareDatabase() {
		// prevents PostgreSQL from getting faster from test execution to test execution
		TestHelper.clearDatabaseCache();
	}

	@AfterClass
	public void writeRuntimes() {
		new SamplePlotter(Aggregator.NONE, RESULTFILE_NAME, sampleRunner, SAMPLE_PROPERTY).writeAll();
		new SamplePlotter(Aggregator.AVERAGE, RESULTFILE_NAME, sampleRunner, SAMPLE_PROPERTY).writeAll();
		new SamplePlotter(Aggregator.MEDIAN, RESULTFILE_NAME, sampleRunner, SAMPLE_PROPERTY).writeAll();
	}

	// Test methods

	@Test(invocationCount = REPEAT_COUNT)
	public void testRuntimes() {
		++currentTestCount;

		LOGGER.trace("  #{} of runtime test executions", currentTestCount);
		LOGGER.trace("-----------------------------");

		// Calculate runtime for algorithms with node expiration (and MDijkstra)
		final Collection<Class<? extends AbstractAlgorithm>> algorithms = TestHelper.getAllAlgorithms(true);
		sampleRunner.sample(TestHelper::runQuietly, algorithms, LOGGER);
	}

}
