package it.unibz.inf.isochrone.config;

import it.unibz.inf.isochrone.TestConfiguration;
import it.unibz.inf.isochrone.config.ConfigAlgorithm.ConfigAlgorithmBuilder;
import it.unibz.inf.isochrone.config.ConfigAlgorithm.Direction;
import it.unibz.inf.isochrone.config.ConfigAlgorithm.Mode;

import java.awt.geom.Point2D;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.Month;
import java.util.Collection;

import org.testng.Assert;
import org.testng.annotations.Test;

public class ConfigAlgorithmTest {
	private static final LocalDateTime DATETIME = LocalDateTime.of(2013, Month.FEBRUARY, 1, 15, 30);
	private static final BigDecimal SPEED =  BigDecimal.valueOf(1.5d);
	private static final Direction DIRECTION = Direction.INCOMING;
	private static final Long DURATION = 50L;
	private static final Mode MODE = Mode.MULTIMODAL;
	private static final Point2D.Double QUERYPOINT = TestConfiguration.DEFAULT_QUERYPOINT;
	private static final ConfigAlgorithm TEST_PARAMETER = new ConfigAlgorithmBuilder(QUERYPOINT).dateTime(DATETIME).direction(DIRECTION).duration(DURATION).mode(MODE).walkingSpeed(SPEED).build();

	// Test methods

	@Test
	public void testDate() {
		Assert.assertEquals(TEST_PARAMETER.getDateTime(), DATETIME, "Time unexpected");
	}

	@Test
	public void testDuration() {
		Assert.assertEquals(TEST_PARAMETER.getDuration(), DURATION, "Duration unexpected");
	}

	@Test
	public void testMode() {
		Assert.assertEquals(TEST_PARAMETER.getMode(), Mode.MULTIMODAL, "Mode unexpected");
	}

	@Test
	public void testQueryPoints() {
		final Collection<Point2D.Double> queryPoints = TEST_PARAMETER.getQueryPoints();

		Assert.assertNotNull(queryPoints, "Null QueryPoint unexpected");
		Assert.assertEquals(queryPoints.size(), 1, "Query point count unexpected");
		Assert.assertEquals(queryPoints.iterator().next(), QUERYPOINT, "Starting point unexpected");
	}

	@Test
	public void testWalkingSpeed() {
		Assert.assertEquals(TEST_PARAMETER.getWalkingSpeed(), SPEED, "WalkingSpeed unexpected");
	}

	@Test
	public void testUseCache() {
		Assert.assertFalse(TEST_PARAMETER.isCacheActive(), "Cache state unexpected");
	}

	@Test
	public void testUseCacheIncremental() {
		Assert.assertFalse(TEST_PARAMETER.isCacheIncrementalActive(), "CacheIncremental state unexpected");
	}

}
