package it.unibz.inf.isochrone.config;

import java.util.Collection;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

public class ConfigDatasetTest {
	private static final Logger LOGGER = LogManager.getLogger(ConfigDatasetTest.class);
	private static final int EXPECTED_NUM_DATASETS = 6;
	private static final int EXPECTED_NUM_DATASETS_FILTERED = 4;
	private static final String[] EXPECTED_DATASETS = new String[] {"bz", "it", "sf", "st"};
	private static final String[] EXPECTED_DATASETS_FILTERED = new String[] {"bz"};

	// Public methods

	@BeforeClass
	public void setup() {
		LOGGER.info("Starting test(s) in class \"{}\"", ConfigDatasetTest.class.getSimpleName());
		LOGGER.info("-----------------------------");
	}

	// Test methods

	@Test
	public void testDatasetCount() {
		final Collection<String> datasetCollection = ConfigDataset.getAvailableDatasets(false);
		Assert.assertEquals(datasetCollection.size(), EXPECTED_NUM_DATASETS, "Unexpected number of configured datasets");
	}

	@Test
	public void testDatasetNames() {
		final Collection<String> datasetCollection = ConfigDataset.getAvailableDatasets(false);
		for (final String dsName : EXPECTED_DATASETS) {
			Assert.assertTrue(datasetCollection.contains(dsName), "Dataset of " + dsName + " unconfigured");
		}
	}

	@Test
	public void testDatasetCountFiltered() {
		final Collection<String> datasetCollectionFiltered = ConfigDataset.getAvailableDatasets(true);
		Assert.assertEquals(datasetCollectionFiltered.size(), EXPECTED_NUM_DATASETS_FILTERED, "Unexpected number of configured datasets");
	}

	@Test
	public void testDatasetNamesFiltered() {
		final Collection<String> datasetCollectionFiltered = ConfigDataset.getAvailableDatasets(true);
		for (final String dsName : EXPECTED_DATASETS_FILTERED) {
			Assert.assertTrue(datasetCollectionFiltered.contains(dsName), "Dataset of " + dsName + " unconfigured");
		}
	}

	@Test
	public void testGetAllDatasets() {
		final Collection<String> datasetCollectionFiltered = ConfigDataset.getAvailableDatasets(true);
		final Collection<String> datasetPublic = ConfigDataset.getAllDatasets();

		Assert.assertEquals(datasetPublic, datasetCollectionFiltered, "Public datasets (retievable by public method getAllDatasets) does not match filtered available datasets");
	}

}
